/*

	Simple program for fitness function for Quantum Circuit synthesis
	worls only for circuits with more than 1 wire.

	using minimization methods to reduce circuits length and cost
	-- possible modes
		- simple fitness function (only error)
		- complex fitness functin (scaled weighted sum of error and cost)
		- pareto replication
		- replicator USS (universal stochastic sampling)
		- roulette wheel

  -- coments to the names of programs
  -- example GS2simple2Mut
		- GS generic name of the program
		- 2 error is evaluated on bassis of both somponents of the matrix real and imaginary parts
		- simple/scaled/pareto type of fitness
		- 2 Mut - mutation operator is applied for each element of the sting representation

  -- all commands ar given through file



*/

#include "stdafx.h"
using namespace std;

typedef complex<double> dComp;


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///here are stred all consecutive matrices of the circuit
struct qMatrix
{
	dComp gateMatrix1[MAXGATEDEM][MAXGATEDEM];	// matrix representing this qGate
	int Cost;
	qMatrix *next;

};

// temporary structures used only in incialization
struct tempT
{
       int numberGates;					// number of gates in this catagory
       char nameGate[MAXNUMOFGATES];	// for a particular number of input and output, the list of the gates can be used
};

// structure holding the quantum gates
struct qGate
{
       char representation;		// using a character to represent it,
       int numIO;									// number of input wires
	   int Cost;									// cost of this gate
		dComp gateMatrix1[MAXGATEDEM][MAXGATEDEM];	// matrix representing this qGate

};
// structure holding the quantum gates
// for calulating fitness
struct qGate1
{
       char representation;		// using a character to represent it,
       int numIO;									// number of input wires
	   int Cost;									// cost of this gate
	   qMatrix *next;
	   int mNumber;
       string my_string;							// function representation as string
	   dComp gateMatrix1[MAXGATEDEM][MAXGATEDEM];	// matrix representing of the complete circuit

};
// structure representing an individual in the GA population
struct Individual
{
       float fitness;		// fitness value
       string my_string;	// function representation as string
	   float Error;			// Error of the circuit
	   int Groups;			// for share fitness calcualtion
	   int Rank;
};


// holds GA functions
class GA
{
public:
	GA(){}
  ~GA(){};
  dComp I;								//complex variable
	// both here bellow fields will 
	// be in the future replaced by
	// array of pointers
  Individual *population[MAXNUMBER];		// population (array of object Individual)
  Individual *new_population[MAXNUMBER];	// population of individuals used for replication ... non optimal to be improved
  qGate *gateArray[MAXNUMOFGATES];			// all agtes are stored in here
  qGate1 *gateArray1[MAXNUMOFGATES];			// allagtes are stored in here
  int numofgates;							// number of gates to be use < MAXNUMOFGATES
  Individual bestIndv;						// best Individual from the run
  ifstream in_stream;						// input file stream
  ofstream out_stream;						// output file stream
  stringstream ss;							// stream for string conversion

  int nResGates;

//////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////
	
  // parameters for the GA
	// size of the population < MAXNUMOFGATES
  int populationNumber;
	// probabilities of mutation and crossover
  double proba_Mutation;
  double proba_Crossover;
	// scaling for the complex fitness function
	// alpha + beta = 1
  float alpha;
  float beta;
	//scaling fo the pareto replication
	//default alpha1 = 1, beta1 = 1
  float alpha1;
  float beta1;
	// estimated minimum of cost for a given circuit
  int divider;
	// type of GA Baldwinian or normal
	//0 - nomral, 1 - baldwinian
  int Ga;
	//setting different types of mutation
	//0 - normal, 1 - bitwise
  int mutation;
	//different types of selection
	//0 - RW, 1- SUS 
  int replicator;
	// type of fitness function
	//0 - simple0, 1 - simple1, 2 - complex0, 3 - complex1
  int fitness;
	//indicator of pareto rplication
	//0 - no, 1 - yes
  bool pareto;
	//Threshold for replication
	//0 - no threshold otherwise must be smaller than 1
  double threshold;
//////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////

  
  
	//methods
  void initializePop();					// initialize the population
  void apply1Crossover(int, int);		// With a 1 point crossover probability recombine 2 parents into 2 new child
  void applyMutation(int);				// With a mutation probability mutate individual either normal or bitwise
  void applyReplication();				// Replication of teh selected individuals either SUS or RW
  void applyParetoReplication();		// Replication for pareto optimal GA
  void makeFitness();					// according to the input it calculates different fitness values based on the error and the cost 
  void repairCircuits();

  void setTerminatingCondition(int);	// somemethod setting final criteria
  bool terminatingCondition();			// some method returning if the final conditions were reached
  void individualMatrix(int, bool);		// calculation of the individual matrix representation for each individual
  int minimizeCirc(Individual*);		// method minimizing circuit ... not used always
  void closeStream();					// method close streams to files
  void output(int);						// writes down the result of the run
  void outputBest(int);					// writes down the best individual from one generation or the allover best
  Individual setIndv(Individual);		// copy one Individual and output it
  void setIndv(Individual*, Individual*); //copy two indidual by their addresses

private:

  qGate nullGate;						// nullGate - wire
  qGate finalGate;						// gate/circuit we are looking for
  tempT stringArray[MAXINDVWIRENUM];	// sorted by input wires in asending order
  int resultnum;						// the number of wire for input and output of the result gate
  int generationCondition;				// termination criterian
  bool condition;						// some boolean condition
  float best;

  qGate tensorProduct(qGate*, qGate*);	// tensor product of gate A and gate B
  qGate matrixProduct(qGate*, qGate*);	// matrix product of gate A and gate B
  int getGate(char);					// retunrs teh char-representation of the gate at teh int index from gateArray
  void reduceSegNumber(int);			// reduce teh total length of the circuit
  void initializeGates();				// reads teh gates from a file and set them in an array
  void makeIndvString(int);				// creates string representation for each individual
  void setStringArray();				// set an array used only for intialization
  bool getFitness();					// looks for the best individual with fitness 1


};



/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;


int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	else
	{


		
		srand( time(NULL) );
		int generationCondition = 0;

		//ignition
		GA g;
		g.initializePop();

		for (int a = 0;a<g.nResGates;a++)
		{
				g.individualMatrix(a,false);
		}

		g.makeFitness();

		cout<<"Initialization completely done"<<endl;
		
		g.setIndv(&(g.bestIndv), g.population[0]);
		int counter = 0;

		//life cycle
		while (!g.terminatingCondition())
		{

			g.makeFitness();
			cout<<" current generation is "<<generationCondition<<endl;
			/*for (a = 0;a<g.populationNumber;a++)
				if (!g.pareto)
					
				else g.applyParetoReplication();
				*/
				g.applyReplication();

			for (a = 0;a<g.populationNumber;a++)
			{
				g.applyMutation(a);
			}

			if (counter == 100)
			{
				g.outputBest(generationCondition);
				counter = 0;
			}


			generationCondition++;
			//cout<<"getting terminating condition"<<endl;
			g.setTerminatingCondition(generationCondition);
			counter++;
		} 

  
		//---------printing---------------
		g.output(generationCondition);
		g.closeStream();
	}
	return nRetCode;
}

/*
	
     calculation of individual fitness!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     and of individual matrix representation
	 setting the fitness for each individual also
	 because each individual stores only a representation 
	 string not the whole circuit

*/
void GA::individualMatrix(int i, bool display)
{    
	int Time, a, b, c;
    //four free gates for operations
    qGate *temp0, *temp1, *finalG;
	qMatrix *qM;
	qGate1 *I;

	Time = 0;
	I = gateArray1[i];
	I->Cost = 0;
	I->next = new qMatrix;
	qM = I->next;
	//out_stream <<" individual "<<i<<endl;

	int begin = 0;
	int end = I->my_string.find("p", begin+1);
		
	while(end > 0) 
	{
		if ((end - begin) > 1)
		{
			for (a = 0;a<= numofgates;a++)
				if (gateArray[a]->representation == I->my_string.at(begin+1)) 
				{
					temp0= gateArray[a];
					begin++; 
					I->Cost+= gateArray[a]->Cost;
					break;
				}
					           
			for (b = begin+1;b<end;b++) 
			{
				for (a = 0;a<= numofgates;a++)
					if (gateArray[a]->representation == I->my_string.at(b)) 
					{
						temp1= gateArray[a];
						I->Cost+= gateArray[a]->Cost;
						break;
					}
				
				temp0 = &tensorProduct(temp0, temp1);
			}

			if (Time == 0) 
			{
				finalG = temp0;
				I->mNumber = 1;
				Time++;
			}
			else 
			{
				I->mNumber+= 1;
				finalG = &matrixProduct(finalG, temp0); 
				Time++;
			}

			qM->Cost = I->Cost;

			for (b = 0; b<(int(pow(2,I->numIO)));b++)
				for (c = 0; c<(int(pow(2,I->numIO)));c++)
				{
					qM->gateMatrix1[b][c] = finalG->gateMatrix1[b][c];
				 }
 
			qM->next = new qMatrix;
			qM = qM->next;
		}
		begin = I->my_string.find("p", end);
		end = I->my_string.find("p", begin+1);
	}

	for (b = 0; b<(int(pow(2,I->numIO)));b++)
	{
		for (c = 0; c<(int(pow(2,I->numIO)));c++)
		{
			I->gateMatrix1[b][c] = finalG->gateMatrix1[b][c];
		 	 //out_stream << I->gateMatrix1[b][c] << " ";
		}
		 //out_stream << endl;
	}
	// out_stream << endl<<endl;

}


void GA::setStringArray()
{
	qGate *Q;
	int count[MAXNUMOFGATES];
	
	for (int j = 0; j < MAXGATEINPUT; j++)
	{
		count[j] = 0;
		for (int i = 0; i < numofgates; i++)
		{
			Q = gateArray[i];
			if (Q->numIO == (j+1))
			{
				stringArray[j].nameGate[count[j]] = Q->representation;
				count[j]++;
			}
			stringArray[j].numberGates = count[j];
		}
	}
}


void GA::makeIndvString(int i)
{
	int tempNumber;
	Individual *I = population[i];
	char t;
	

	I->my_string = "";
	I->my_string.append(1, char('0'));
	I->my_string.append(1, '.');
	for (int j = 2; j < MAXSTRING; j++) // for each segment of this individual
	{
		tempNumber = rand()%9;
		t = char('0'+tempNumber);
		if (j == MAXSTRING/2)
			I->my_string.append(1, char('0'));
		else if (j == MAXSTRING/2+1)
			I->my_string.append(1, '.');
		else 
			I->my_string.append(1, t);
	}
}




void GA::output(int generationCondition)
{
	qGate *qG;
	qGate1 *qG1;
	qMatrix *qM;

     out_stream << "There are " << numofgates << " gates from the input file.\n";
     for (int i = 0; i < numofgates; i++)
     {
		 qG = gateArray[i];

         out_stream << "The gate number " << i << " is: " << endl;
         out_stream << "And the gate representation is: " << gateArray[i]->representation << endl;
         out_stream << "Number of inputs is: " << qG->numIO << endl;
         out_stream << "Cost of the gate is: " << qG->Cost  << endl;

         for (int j = 0; j < (int(pow(2,qG->numIO))); j++)
         {
             for (int k = 0; k < (int(pow(2,qG->numIO))); k++)
             {
                 out_stream << qG->gateMatrix1[j][k] << " ";
             }
             out_stream << endl;
         }
     }
     out_stream << "-------------------------------------------\n";
     out_stream << "-------------------------------------------\n";
     out_stream << "-------------------------------------------\n";

     out_stream << "There are " << nResGates << " final gates.\n";
     for (i = 0; i < nResGates; i++)
     {
		 qG1 = gateArray1[i];
		 qM = qG1->next;

         out_stream << "The gate number " << i << " is: " << endl;
         out_stream << "And the gate representation is: " << qG1->my_string << endl;
         out_stream << "Number of inputs is: " << qG1->numIO << endl;
         out_stream << "Cost is: " << qG1->Cost << endl;
         out_stream << "Circuit has : " << qG1->mNumber <<" of matrices "<< endl;


		 for (int v = 0;v<qG1->mNumber;v++)
		 {			 
			out_stream << "First address : " << qM << endl;
			 for (int j = 0; j < (int(pow(2,qG1->numIO))); j++)
			 {
				 for (int k = 0; k < (int(pow(2,qG1->numIO))); k++)
				 {
					 out_stream << qM->gateMatrix1[j][k] << " ";
				 }
				 out_stream << endl;
			 }
 			 qM = qM ->next;
		 }
	 }


      out_stream << "---------------------------" << endl;
      out_stream << " End of run at " <<generationCondition<< endl;
      out_stream << "---------------------------" << endl;
      out_stream << "----- Best Individual -----" << endl;

	  out_stream<<endl;
	  out_stream<< bestIndv.my_string<<" representation "<<endl;
	  out_stream<< bestIndv.fitness<<" fitness "<<endl;
}


void GA::outputBest(int generationCondition)
{
	setIndv(population[0], &bestIndv);
	individualMatrix(0, true);
	
	out_stream << "---------------------------" << endl;
	out_stream << " End of run at " <<generationCondition<< endl;
	out_stream << "---------------------------" << endl;
	out_stream << "----- Best Individual -----" << endl;
	out_stream<<endl;
	out_stream<< bestIndv.my_string<<" representation "<<endl;
	out_stream<< bestIndv.fitness<<" fitness "<<endl;
	out_stream<< bestIndv.Error<<" error "<<endl;
}

// Restrictions:
// 1. no fan out - ex, 5 inputs has to have 5 outputs
// 2. maximum 5 I/O (geometric increase of complexity with more I/Os)
void GA::initializePop()
{
     char inFile[100];
     char outFile[100];
     condition= false;

     // Open input and output files.
     cout << "Please input the source file" << endl;
     cin >> inFile;


     in_stream.open(inFile);
     if (in_stream.fail())
     {
        cout << "Input file opening failed.\n";
        exit(1);
     }

     cout << "\nPlease input the output file" << endl;
     cin >> outFile;

     out_stream.open(outFile);
     if (out_stream.fail())
     {
        cout << "Output file opening failed.\n";
        exit (1);
     }

     initializeGates();
     setStringArray();

     for (int i = 0; i < populationNumber; i++)
     {
		 population[i] = new Individual;
		 new_population[i] = new Individual;	
		 makeIndvString(i);
     }
    out_stream << "\nintialization done" << endl;

}



int GA::getGate(char b)
{
	for (int a = 0;a<numofgates;a++)
		if (gateArray[a]->representation == b) return a;
  
}


// Restrictions:
// 1. mutate one gate to another with same number of I/O
// 2. mutate 2 or more parallel connections between them
// 3. nutate any gate with n I/O to any gates or gate with same n I/O
void GA::applyMutation(int indv)
{
	//cout<<population[indv]->my_string<<endl;

	int proba1, proba2, length;
	float proba;
	Individual *I = population[indv];

	///////////////////////
	//setting mutation for either bitwise either normal
	if (mutation != 0)
		length = I->my_string.length();
	else 
		length = 1;
	///////////////////////
	///////////////////////

	for (int A = 0; A<length;A++)
	{
		proba  = 1-(double(1.0*rand()/((double)RAND_MAX+1.0)));
		
		if (proba <= proba_Mutation)
		{
			do {
				proba2 =  rand()%(*I).my_string.length();
			}while ( proba2 < I->my_string.length() && I->my_string.at(proba2) == '.');
			 
			proba1 = rand()%9;
			
			I->my_string.erase(proba2, 1);
			I->my_string.insert(proba2, 1, char('0'+proba1));
		}
	}
}



// Restrictions:
// 1. replace gate with same number of I/O
// 2. replace one gate with n I/O with any combination of smaller gates with
//    same number of I/O
// 3. conserve strict correspondence of connections
//                                void operator=(const class bla & other) { mymember=other.mymember; }
void GA::apply1Crossover(int ind1, int ind2)
{  
     //	apply rules and make crossover
	int proba1;
	Individual *I = new_population[ind1];
	Individual *J = new_population[ind2];
	
	do {
		proba1 = (rand()%(I->my_string.length()));
	}while(proba1 >= I->my_string.length());
	
	basic_string<char> temp1;
	basic_string<char> temp2;
	
	temp1 = I->my_string.substr(proba1);
	temp2 = J->my_string.substr(proba1);
	
	if (temp1.length() > 1 && temp2.length() > 1)
	{
		I->my_string.replace(proba1, I->my_string.length(), temp2, 0, temp2.length());
		J->my_string.replace(proba1, J->my_string.length(), temp1, 0, temp1.length());
	}

}


void GA::applyReplication()
{ 

    float fitnessSum = 0;
	float first, second, tempsum, proba;
	int indv1, indv2;

    for (int i = 0; i < populationNumber; i++)
		fitnessSum += population[i]->fitness;

    int counter = 0;
    bool condition = false;
    while (!condition)
	{
		if (replicator == 0)
		{
			//roulette wheel
			if (threshold > 0)
				do{
					indv1  = rand()%(populationNumber-1);
					indv2  = rand()%(populationNumber-1);
				} while(population[indv1]->fitness > threshold &&  population[indv2]->fitness > threshold);
			else
				{
					first  = (1-(double(1.0*rand()/((double)RAND_MAX+1.0))))*fitnessSum;
					second  = (1-(double(1.0*rand()/((double)RAND_MAX+1.0))))*fitnessSum;
				}
		}
		else
		{
			//stochastic uniersal sampling
			if (threshold > 0)
				do{
					indv1  = rand()%(populationNumber-1);
					indv2  = rand()%(populationNumber-1);
				} while(indv1 > threshold &&  indv2 > threshold);
			else
				{
					first  = ((1-(double(1.0*rand()/((double)RAND_MAX+1.0))))*fitnessSum)/2;
					second = fitnessSum - first;
				}
		}

		if (threshold == 0)
		{
			tempsum = 0;
			for (int c = 0;c<populationNumber;c++)
			{
				tempsum += population[c]->fitness;
				if (tempsum >= first) 
				{
					indv1 = c; 
					break;
				}
			}
			
			tempsum = 0;
			for (int d = 0;d<populationNumber;d++)
			{
				tempsum += population[d]->fitness;
				if (tempsum >= second) 
				{
					indv2 = d; 
					break;
				}
			}
		}
		
		if(counter == populationNumber-1)
		{
			setIndv(new_population[counter], population[indv1]);
			counter++;
		} else
		{
			setIndv(new_population[counter], population[indv1]);
			setIndv(new_population[counter+1], population[indv2]);

			proba  = 1-(double(1.0*rand()/((double)RAND_MAX+1.0)));
			if (proba < proba_Crossover && indv1 != indv2)
			{
				apply1Crossover(counter, counter+1);
			}
			
			counter += 2;
		}
		if (counter >= populationNumber-1) condition = true;
	}

    for (i = 0;i<populationNumber;i++)
	{
	   setIndv(population[i], new_population[i]);
	 
	}
	repairCircuits();
}

//below is the method for pareto replicatin
//you are not forced to it
//if you want to se it iot is on you to modify to 
//change it in order it satisfies your needs

/*
void GA::applyParetoReplication()
{ 
	float proba;
	int indv1, indv2, tempsum, fitnessSum, first, second;
	int counter = 0;
    bool condition = false;

	for (int i = 0; i < populationNumber; i++) population[i]->Rank = -1;
	int temp_rank;
	
	for (i = 0; i < populationNumber; i++)
    {
		temp_rank = 0;
		for (int j = 0; j < populationNumber; j++)
		{
			if (population[i]->Rank <= -1)
			{
				if ((alpha1*population[i]->Error) <= population[j]->Error && (beta1*population[i]->Cost) <= population[j]->Cost)
					temp_rank++;
			}
		}
		population[i]->Rank = temp_rank;
    }

	fitnessSum = 0;
	for (i = 0; i < populationNumber; i++)
    {
         fitnessSum += population[i]->Rank;
    }



    while (!condition)
	{
		if (replicator == 0)
		{
			//roulette wheel
			if (threshold > 0)
				do{
					indv1  = rand()%(populationNumber-1);
					indv2  = rand()%(populationNumber-1);
				} while(population[indv1]->fitness > threshold &&  population[indv2]->fitness > threshold);
			else
				{
					first  = (1-(double(1.0*rand()/((double)RAND_MAX+1.0))))*fitnessSum;
					second  = (1-(double(1.0*rand()/((double)RAND_MAX+1.0))))*fitnessSum;
				}
		}
		else
		{
			//stochastic uniersal sampling
			if (threshold > 0)
				do{
					indv1  = rand()%(populationNumber-1);
					indv2  = rand()%(populationNumber-1);
				} while(indv1 > threshold &&  indv2 > threshold);
			else
				{
					first  = ((1-(double(1.0*rand()/((double)RAND_MAX+1.0))))*fitnessSum)/2;
					second = fitnessSum - first;
				}
		}

		if (threshold == 0)
		{
			tempsum = 0;
			for (int c = 0;c<populationNumber;c++)
			{
				tempsum += population[c]->Rank;
				if (tempsum >= first) 
				{
					indv1 = c; 
					break;
				}
			}
			
			tempsum = 0;
			for (int d = 0;d<populationNumber;d++)
			{
				tempsum += population[d]->Rank;
				if (tempsum >= second) 
				{
					indv2 = d; 
					break;
				}
			}
		}
		

		if(counter == populationNumber-1){
			  setIndv(new_population[counter], population[indv1]);
			counter++;
		} else
		{

			 setIndv(new_population[counter], population[indv1]);
			 setIndv(new_population[counter+1], population[indv2]);

			proba  = 1-(double(1.0*rand()/((double)RAND_MAX+1.0)));
			if (proba < proba_Crossover && indv1 != indv2)
				if (crossover == 1)
					apply1Crossover(counter, counter+1);
				else apply2Crossover(counter, counter+1);

			counter += 2;
		}
		if (counter >= populationNumber-1) condition = true;
	}
    for (i = 0;i<populationNumber;i++)
	   setIndv(population[i], new_population[i]);
}

*/
void GA::setIndv(Individual *indv1, Individual *indv2)
{
		indv1->fitness = indv2->fitness;
		indv1->Error = indv2->Error;
		indv1->my_string = indv2->my_string;
}

void GA::repairCircuits()
{
	Individual *I;
	int temp;

	for (int a = 0;a<populationNumber;a++)
	{
		I = population[a];
		int v = I->my_string.length();

		for (int b = 0; b<MAXSTRING;b++)
		{
			if (b > v)
			{
				if (b == 1 || b == 17)
				{
					if (I->my_string.at(b) != '.')
					{
						I->my_string.erase(b,1);
						I->my_string.insert(b, ".");
					}
				}
				else
					if (b == 0 || b == 16)
					{
						if (I->my_string.at(b) != '0')
						{
							I->my_string.erase(b,1);
							I->my_string.insert(b, "0");
						}
					}
					else
					{
						temp = rand()%9;
						I->my_string.append(1, char('0'+temp));
					}
			}
			else
				if (b == 1 || b == 17)
				{
					if (I->my_string.at(b) != '.')
					{
						I->my_string.erase(b,1);
						I->my_string.insert(b, ".");
					}
				}
				else
					if (b == 0 || b == 16)
					{
						if (I->my_string.at(b) != '0')
						{
							I->my_string.erase(b,1);
							I->my_string.insert(b, "0");
						}
					}
					else
						if (I->my_string.at(b) == '.')
						{
							temp = rand()%9;
							I->my_string.erase(b,1);
							I->my_string.insert(b+1, 1, char('0'+temp));
						}

		}
	}
}



void GA::makeFitness()
{
	Individual *I;
	qGate1 *qG1;
	qMatrix *qM;
	double Error, Cost, fitness, fitness0, fitness1;
	
	for (int a = 0;a<populationNumber;a++)
	{
		I = population[a];

		basic_string<char> tempstr = I->my_string.substr(0,I->my_string.length()/2);
		basic_string<char> tempstr1 = I->my_string.substr(I->my_string.length()/2,I->my_string.length());
		//example of how the evolved individual represents coefficients alpha and beta
		char   c[16];

		//cout<< I->my_string <<"   "<< a<<endl;
		// Get the string into a char array.
		ss << tempstr << endl;
		ss >> c;
		
		// Get the string into a float.
		ss << tempstr << endl;
		ss >> alpha;
	
		//cout << I->my_string.substr(I->my_string.length()/2,I->my_string.length()) << endl;
		//cout << tempstr << endl;
		//cout <<  alpha << endl;
	
	
		// Get the string into a char array.
		ss << tempstr1 << endl;
		ss >> c;
	
		// Get the string into a float.
		ss << tempstr1 << endl;
		ss >>  beta;
	
		//cout << c << endl;
		//cout << tempstr1 << endl;
		//cout << beta << endl;

		I->fitness = 0;
		I->Error = 0;
		fitness = 0;
		Error = 0;
		Cost = 0;
		fitness1 = 0;
		fitness0 = 0;
		//cout <<" gates "<<nResGates<< endl;


		for (int b = 0;b<nResGates;b++)
		{
			qG1 = gateArray1[b];
			qM = qG1->next;
		
			//cout <<" fitness "<<fitness<< endl;

			for (int c = 0;c<qG1->mNumber;c++)
			{
				for (int j = 0; j < (int(pow(2,qG1->numIO))); j++)
				{
					for (int k = 0; k < (int(pow(2,qG1->numIO))); k++)
						if (qG1->gateMatrix1[j][k] != qM->gateMatrix1[j][k]) Error++;
				}
				

				//exa,ple of fitness function
				//here the evolution is based on the coefficients alpha and beta
				Error =(Error / pow(2,(qG1->numIO*2)));
				Error = 1/(Error+1);
				I->Error += Error;
			//cout <<" error "<<Error<< endl;
				Cost = qG1->Cost;
				Cost = exp(-pow((divider-Cost),2));
			//cout <<" cost "<<Cost<< endl;

				if (c == 0)
					 fitness0 = alpha*Error+beta*Cost;
				else
				{
					fitness1 = alpha*Error+beta*Cost;

					if (fitness0 < fitness1)
						fitness++;
					fitness0 = fitness1;
				}				
				 
				qM  = qM->next;
			}
			I->Error /= (qG1->mNumber);
			fitness /= (qG1->mNumber);
		}
		fitness /=nResGates;
		
		I->Error /= nResGates;
		I->fitness = fitness;

		//cout <<" fitness of individual "<<a<<"  "<<I->fitness<< endl;

	}

}

bool GA::terminatingCondition()
{
     return condition;
}

void GA::setTerminatingCondition(int counter)
{
     if (counter >= MAXGEN)
	 {
		 out_stream<<" generation max reached "<<endl<<endl<<endl;
		 condition = true;
	 }
     else  if (getFitness()) 
			{
				out_stream<<" Solution found "<<endl<<endl<<endl;
				condition = true;
			}	
			else condition = false;
}



bool GA::getFitness()
{
	for (int a = 0;a<populationNumber;a++)
	{	
		if (population[a]->Error <= 0)
		{
			out_stream<<" best individual is "<<a<<endl<<endl;
			setIndv(&bestIndv, population[a]);
			return true;
		}
		else
			if (population[a]->fitness >= bestIndv.fitness)
			{
				setIndv(&bestIndv, population[a]);
			}
			else 
				if (population[a]->Error <= bestIndv.Error)
					setIndv(&bestIndv, population[a]);
	}
	return false;
}


void GA::initializeGates()
{
	I = dComp (0., 1.); //  Declare complex double variables
	double x;
	int counter = 0;
	string temp;
	
	//reading the input file 
	//first parameters
	in_stream>> populationNumber,"%\n";		//int
	in_stream >> proba_Mutation;		//double
	in_stream >> proba_Crossover;		//double
	in_stream >> alpha1;				//float
	in_stream >> beta1;					//float
	in_stream >> divider;				//int
	//reading file divider
	in_stream >> temp;

	in_stream >> Ga;					//int
	in_stream >> mutation;				//int
	in_stream >> replicator;			//int
	in_stream >> pareto;				//bool
	in_stream >> threshold;				//double
	//reading file divider
	in_stream >> temp;

	out_stream<< " output file of synthesis  "<<endl;
	out_stream<< " size of the population					"<<populationNumber<<endl;
	out_stream<< " mutation probability is					"<< proba_Mutation<<endl;
	out_stream<< " mutation of crossover is					"<<proba_Crossover<<endl;
	out_stream<< " factor Alpha1 is							"<<alpha1<<endl;
	out_stream<< " factor Beta1 is							"<<beta1<<endl;
	out_stream<< " estimated minimum cost of this circuit	"<<divider<<endl;
	out_stream<< " -------------------------------------- "<<endl;
	out_stream<< " type of GA 0 - normal, 1 - Darwinian                      "<<Ga<<endl;
	out_stream<< " type of mutation 0 - normal, 1 - bitwise                  "<<mutation<<endl;
	out_stream<< " type of replication 0 - RW, 1 - SUS                       "<<replicator<<endl;
	out_stream<< " Pareto optimization 0 - no, 1 - yes                       "<< pareto<<endl;
	out_stream<< " Threshold replication 0 - no, other - threshold           "<<threshold<<endl;



	//bedining reading the gates
    in_stream >> numofgates;
	numofgates++;

	nullGate.numIO = 1;
	nullGate.representation = 'W';
	nullGate.gateMatrix1[0][0] = 1;
	nullGate.gateMatrix1[0][1] = 0;
	nullGate.gateMatrix1[1][0] = 0;
	nullGate.gateMatrix1[1][1] = 1;
	nullGate.Cost = 0;
	gateArray[0] = &nullGate;


	// get first gate
    for (int a = 1; a < numofgates; a++)
	{
		gateArray[a] = new qGate;
		//cout<<gateArray[a]<<endl;
		
		in_stream >> gateArray[a]->numIO;
		gateArray[a]->representation = char(a+65); // starting from A
		in_stream >> gateArray[a]->Cost;
		
		for (int j = 0; j < (int(pow(2,gateArray[a]->numIO))); j++)
		{
			for (int k = 0; k < (int(pow(2,gateArray[a]->numIO))); k++)
			{
				in_stream >> x;
				if (x == 2)
					gateArray[a]->gateMatrix1[j][k] = I;
				else if (x == -2)
					gateArray[a]->gateMatrix1[j][k] = -I;
				else if (x == 4)
					gateArray[a]->gateMatrix1[j][k] = exp(complex<double>(0, (3.1416/4)));
				else if (x == 5)
					gateArray[a]->gateMatrix1[j][k] = complex<double>(0.5, 0.5);
				else if (x == -5)
					gateArray[a]->gateMatrix1[j][k] = complex<double>(0.5, -0.5);
				else gateArray[a]->gateMatrix1[j][k] = x;
			}
		}
		counter++;
	}
	//reading file dividers
	in_stream >> temp;
	in_stream >> temp;
	in_stream >> temp;
	in_stream >> nResGates;
	// get the expected result matrix from the file


    for (a = 0; a < nResGates; a++)
	{
		in_stream >> resultnum;
		gateArray1[a] = new qGate1;
		gateArray1[a]->numIO = resultnum;
		in_stream >> gateArray1[a]->my_string;
	

	}
	out_stream<< "------------------------------------------"<<endl;  
	out_stream<< "------------------------------------------"<<endl;  

}


qGate GA::tensorProduct(qGate *A, qGate *B)
{
    qGate pGate;
	for (int i = 0; i < (int(pow(2,A->numIO))); i++)
	{    
		for (int j = 0; j < (int(pow(2,A->numIO))); j++)
		{   
			for (int k = 0; k < (int(pow(2,B->numIO))); k++)
			{   
				for (int l = 0; l < (int(pow(2,B->numIO))); l++)
				{  
					pGate.gateMatrix1[i*(int(pow(2,B->numIO)))+k][j*(int(pow(2,B->numIO)))+l] = A->gateMatrix1[i][j] * B->gateMatrix1[k][l];
				}
			}
		}
	}
	pGate.numIO = (A->numIO + B->numIO);
	return pGate;
}


qGate GA::matrixProduct(qGate *A, qGate *B)
{
	qGate pGate;
	//rows of the first gate
	for (int i = (int(pow(2,A->numIO))-1);i>=0; i--)
	{   //colums of the second gate
		for (int j = 0; j < (int(pow(2,B->numIO))); j++)
		{   //colums of the first matrix
			int l = (int(pow(2,A->numIO))); 
			for (int k = 0; k < (int(pow(2,A->numIO))); k++)
			{ //rows of the second
				pGate.gateMatrix1[j][i] = pGate.gateMatrix1[j][i] + A->gateMatrix1[k][i] * B->gateMatrix1[j][k];
				l--;
			}
		}
	}
	pGate.numIO = A->numIO;
	return pGate;
}

void GA::closeStream()
{
     in_stream.close();
     out_stream.close();
}
